# google-custom-search - adapter

from typing import Any, List

from abc import ABCMeta, abstractmethod

from requests import Session
try:
    from aiohttp import ClientSession
except ImportError:
    async_mode = False
else:
    async_mode = True

from .errors import AsyncError, ApiNotEnabled
from .types import Item


class BaseAdapter(metaclass=ABCMeta):
    """This is the base class for adapters.

    Args:
        apikey (str): Insert google custom search api key.
        engine_id (str): Insert google custom search engine id.

    Attributes:
        APIURL (str): Google Custom Search API URL
    """

    APIURL = "https://www.googleapis.com/customsearch/v1"
    session: Any = None

    def __init__(self, apikey: str, engine_id: str):
        self.apikey = apikey
        self.engine_id = engine_id

    @abstractmethod
    def request(self, method: str, path: str, *args, **kwargs) -> Any:
        ...

    @abstractmethod
    def search(self, *args, **kwargs) -> List[Item]:
        ...

    def _from_dict(self, data: dict) -> List[Item]:
        if data.get('error'):
            raise ApiNotEnabled(
                data['error']['code'], data['error']['message'])
        else:
            return [Item(i) for i in data["items"]]

    def _payload_maker(
        self, query: str, *,
        safe: bool = False,
        filter_: bool = False
    ) -> dict:
        payload = {
            "key": self.apikey,
            "cx": self.engine_id,
            "q": query
        }
        if safe:
            payload["safe"] = "active"
        if not filter_:
            payload["filter"] = 0
        return payload


class RequestsAdapter(BaseAdapter):
    "This class is requestsadapter for sync mode."

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.__session = Session()

    def request(self, method: str, path: str, *args, **kwargs) -> dict:
        return self.__session.request(
            method, self.APIURL + path, *args, **kwargs
        ).json()

    def search(self, *args, **kwargs) -> List[Item]:
        return self._from_dict(
            self.request(
                "GET", "/", params=self._payload_maker(*args, **kwargs)
            )
        )


class AiohttpAdapter(BaseAdapter):
    "This class is aiohttpadapter for async mode."

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        if not async_mode:
            raise AsyncError(
                "This adapter use aiohttp, so please install aiohttp")
        self.__session = ClientSession(*args, **kwargs)

    async def request(self, method: str, path: str, *args, **kwargs) -> dict:
        async with self.__session.request(
            method, self.APIURL + path, *args, **kwargs
        ) as r:
            return await r.json()

    async def search(self, *args, **kwargs) -> List[Item]:
        r = await self.request(
            "GET", "/", params=self._payload_maker(*args, **kwargs)
        )
        return self._from_dict(await r.json())



# google-custom-seaerch - search

from typing import List

from .types import Item
from .adapter import BaseAdapter


class CustomSearch:
    """This is the class used when using Google Custom Search.

    Args:
        adapter (BaseAdapter): Insert adapter
    """
    APIURL: str = "https://www.googleapis.com/customsearch/v1"

    def __init__(self, adapter: BaseAdapter):
        self.adapter = adapter

    def search(self, *args, **kwargs) -> List[Item]:
        """This is searched using api.

        Args:
            query (str): Search keyword
            safe (bool): Using safe mode
            filter_ (filter): Use filter mode

        Returns:
            List[Item]: return result

        Raises:
            ApiNotEnabled: api is not invalid
        """
        return self.adapter.search(*args, **kwargs)



import setuptools


with open("README.md", "r") as fh:
    long_description = fh.read()


def _requires_from_file(filename):
    return open(filename, encoding="utf8").read().splitlines()


def _get_version(filename):
    with open(filename, "r") as f:
        lines = f.readlines()
    version = None
    for line in lines:
        if "__version__" in line:
            version = line.split()[2]
            break
    return version.replace('"', '')


extras_require = {
    "async": [
        "aiohttp>=3.8.1"
    ]
}

setuptools.setup(
    name="google-custom-search",
    version=_get_version("google_custom_search/__init__.py"),
    author="mc-fdc-dev",
    author_email="masato190411@gmail.com",
    description="This is for google custom search api.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/mc-fdc-dev/google-custom-search",
    install_requires=_requires_from_file('requirements.txt'),
    extras_require=extras_require,
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3.8",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
)



from enum import Enum


class Lang(Enum):
    AR = "lang_ar"
    BG = "lang_bg"
    CA = "lang_ca"
    CS = "lang_cs"
    DA = "lang_da"
    DE = "lang_de"
    EL = "lang_el"
    EN = "lang_en"
    ES = "lang_es"
    ET = "lang_et"
    FI = "lang_fi"
    FR = "lang_fr"
    HR = "lang_hr"
    HU = "lang_hu"
    ID = "lang_id"
    IS = "lang_is"
    IT = "lang_it"
    IW = "lang_iw"
    JA = "lang_ja"
    KO = "lang_ko"
    LT = "lang_lt"
    LV = "lang_lv"
    NL = "lang_nl"
    NO = "lang_no"
    PL = "lang_pl"
    PT = "lang_pt"
    RO = "lang_ro"
    RU = "lang_ru"
    SK = "lang_sk"
    SL = "lang_sl"
    SR = "lang_sr"
    SV = "lang_sv"
    TR = "lang_tr"
    ZH_CN = "lang_zh-CN"
    zh_TW = "lang_zh-TW"
